<?php  if ( ! defined('BASEPATH')) exit('No direct script access allowed');
/**
 * Helper pomagający w nawiązaniu sesji umożliwiającej autentykację w portalu ePUAP przy pomocy protokołu SAML 2.0
 * 
 * @author Extern
 * Metody wymagają ustawienia następujacych wartości w pliku konfiguracji aplikacji
 * 
 * $config['urlSecurityServer'] = 'https://hetmantest.epuap.gov.pl/DracoEngine2/draco.jsf';  //adres url serwera bezpieczeństwa Draco
 * $config['urlSPServer'] = 'http://url_do_naszej_stony_odbierajacej_SAMLart_i_wysylajacej_ArtifactResolve_w_celu_pobrania_TGSID';		//adres url	SP czyli skryptu który u nas rozumie odpowiedz SAMLart
 * $config['appID'] = '';											//identyfikator aplikacji w konsoli Draco
 * 
 * Rozwiazywanie artefaktu
 * $config['urlEngineSAMLArtifact'] = 'https://hetmantest.epuap.gov.pl/axis2/services/EngineSAMLArtifact';  //adres uslugi do odbierania zapytania typu ArtifactResolve wysyłanego protokolem SOAP
 *
 * COAP HTTP proxy, jeśli nie będą te zmienne ustawione lub będą miały wartość pustych stringów parametry proxy nie zostaną wpisane do ustawień clienta SOAP (w przypadku bezpośredniego połączenia tak należy to skonfigurować)
 * $config['proxyServer'] = '';					//adres serwera proxy w organizacji
 * $config['proxyPort'] = 8080;								//port serwera proxy
 * $config['proxyUser'] = '';						//użytkownik
 * $config['proxyPassword'] = '';					//hasło

 */
if ( ! function_exists('getUniqueID'))
{
	/**
	 * 
	 * Generacja unikalnego identyfikatora umieszczanego jako atrybut ID w zapytaniach typu AuthnRequest i ArtifactResolve
	 * @param number $length rzeczywista liczba oznaczająca długość wygenerowanego identyfikatora
	 * @return unikalny identyfikator
	 */
    function getUniqueID($length) {
      $chars = "abcdef0123456789";
      $chars_len = strlen($chars);
      $uniqueID = "";
      for ($i = 0; $i < $length; $i++)
        $uniqueID .= substr($chars,rand(0,15),1);
      return "_".$uniqueID;
    }
}
if ( ! function_exists('getTimestamp'))
{
	/**
	 * 
	 * Funkcja do generacji znacznika czasu wykorzystywanego w zapytaniach np. jako atrybut IssueInstant
	 * @param integer $interval wielkość przesunięcia w sekundach jaka zostanie dodana do czasu aktualnego (może być ujemna)
	 * @return sformatowany w formacie TimeStamp znacznik czasu
	 */
    function getTimestamp($interval=0) {
      date_default_timezone_set('Europe/Warsaw');
      return strftime("%Y-%m-%dT%H:%M:%SZ",mktime()+$interval);
	  }
}
if ( ! function_exists('getSAMLRequestUrl'))
{
	/**
	 * 
	 * Metoda tworząca zapytanie typu AuthnRequest (o autoryzację) oraz zwracajaca pełny url z zakodowanym właśniwie parametrem "SAMLRequest"
	 * Metoda wymaga ustawienia właściwych parametrów konfiguracyjnych w pliku konfiguracji aplikacji
	 * @author Extern
	 * @return URL do serwera bezpieczeństwa z wypełnionym parametrem SAMLRequest
	 * @uses $this->load->helper('saml'); 
	 * @uses header('Location: '.getSAMLRequestUrl());
	 */
	function getSAMLRequestUrl()
	{
			$CI =& get_instance();
			$SAMLRequest="<samlp:AuthnRequest xmlns:samlp=\"urn:oasis:names:tc:SAML:2.0:protocol\" ID=\"".getUniqueID(20)."\" Version=\"2.0\" "
			."IssueInstant=\"".getTimestamp()."\" "
			."Destination=\"".$CI->config->item('urlSecurityServer')."\" "
			."IsPassive=\"false\" "
			."AssertionConsumerServiceURL=\"".$CI->config->item('urlSPServer')."\">"
			."<saml:Issuer xmlns:saml=\"urn:oasis:names:tc:SAML:2.0:assertion\">".$CI->config->item('appID')."</saml:Issuer>
			</samlp:AuthnRequest>";
	
			return $CI->config->item('urlSecurityServer').'?SAMLRequest='.addslashes(rawurlencode(base64_encode(gzdeflate($SAMLRequest))));		
	}
}
if ( ! function_exists('getTGSID'))
{
	/**
	 * 
	 * Metoda do wysyłania do serwera (protokołem SAML) zapytania typu "ArtifactResolve" czyli pytania o asercję czyli w przypadku ePUAPu o identyfikator TGSID
	 * Dodatkowo metoda ustawia "cookie" o nazwie TGSID zawierające pobrany identyfikator sesji (jeśli się udało pobrać)
	 * Metoda wymaga obecności w katalogu "libraries" biblioteki "nusoap".
	 *   
	 * @author Extern
	 * @param string $SAMLart referencja do artefaktu uzyskana jako parametr SAMLart po wysłaniu zapytania typu "AuthnRequest"
	 * @param ref strign $msg referencja do zmiennej typu string z przeznaczeniem na komunikaty błędu wygenerowane w czasie trwania metody (metoda nie rzuca wyjątków)
	 * @return Zwraca obiekt typu ArrayObject zawierający właściwości "TGSID" jako string i "expire" jako timestamp czasu ważności ciasteczka. w przypadku niepowodzenia zamiast obiektu zwraca FALSE w. Jeśli metoda się nie powiedzie w zmiennej $msg powinien znaleść się komunikat błędu.
	 * @uses if(isset($_GET['SAMLart']))
     * @uses {
     * @uses 	$this->load->model('admin');
     * @uses 	$msg = '';
     * @uses	$TGSID = getTGSID($_GET["SAMLart"],$msg);
     * @uses }
 
	 */
	function getTGSID($SAMLart,&$msg)
	{
		$CI =& get_instance();
		$CI->load->library('nusoap');
		
		$artifactResolve="<samlp:ArtifactResolve ID=\"".getUniqueID(20)."\" "
		."IssueInstant=\"".getTimestamp()."\" Version=\"2.0\" "
		."xmlns:samlp=\"urn:oasis:names:tc:SAML:2.0:protocol\">"
		."<saml:Issuer xmlns:saml=\"urn:oasis:names:tc:SAML:2.0:assertion\">".$CI->config->item('appID')."</saml:Issuer>"
		."<samlp:Artifact>".$SAMLart."</samlp:Artifact>"
		."</samlp:ArtifactResolve>";
		$client=new nusoap_client($CI->config->item('urlEngineSAMLArtifact'));
		$client->soap_defencoding = 'UTF-8';
		$client->decode_utf8 = true;
		$client->http_encoding = 'UTF-8';
		$client->xml_encoding = 'UTF-8';
		if($CI->config->item('proxyServer') && $CI->config->item('proxyServer') != '' && $CI->config->item('proxyUser') && $CI->config->item('proxyUser') != '' && $CI->config->item('proxyPassword') && $CI->config->item('proxyPassword') != '')
		{
			$proxyPort = 8080;
			if($CI->config->item('proxyPort') && $CI->config->item('proxyPort') != '')
				 $proxyPort = $CI->config->item('proxyPort');
			$client->setHTTPProxy($CI->config->item('proxyServer'),$proxyPort,$CI->config->item('proxyUser'),$CI->config->item('proxyPassword'));
		}
		$result = $client->call('artifactResolve', $artifactResolve,'http://tempuri.org',"artifactResolve",false,null,'document');
		if($client->fault)
		{
			$msg = $client->fault;
			return false;
		}
		else
		{
			if ($client->getError())
            {			
                $msg = $client->getError();
				return false;
            }
            else
            {
				if(isset($result))
				{
					if($result['Response']['Status']['StatusCode']['!Value'] =='urn:oasis:names:tc:SAML:2.0:status:Success')
					{
						setcookie("TGSID", substr($result['Response']['Assertion']['!ID'],1,strlen($result['Response']['Assertion']['!ID'])), strtotime($result['Response']['Assertion']['Subject']['SubjectConfirmation']['SubjectConfirmationData']['!NotOnOrAfter']),'/');
						return new ArrayObject(array("TGSID"=>substr($result['Response']['Assertion']['!ID'],1,strlen($result['Response']['Assertion']['!ID'])),"expire"=>strtotime($result['Response']['Assertion']['Subject']['SubjectConfirmation']['SubjectConfirmationData']['!NotOnOrAfter'])));  
					}
					else
					{
						$msg = "zapytanie nie zakończylo sie sukcesem";
						return false;
					}
				}
			}
		}
		unset($client);				
	}
}
/* End of file saml_helper.php */
/* Location: ./system/helpers/saml_helper.php */